local addonInfo, privateVars = ...

if not EnKai then EnKai = {} end
if not EnKai.inventory then EnKai.inventory = {} end

privateVars.invManager = false

function EnKai.inventory.init ()

	if privateVars.invManager == true then return end
	
	if EnKai.internal.checkEvents ("EnKai.InventoryManager", true) == false then return nil end
	
	if EnKaiInv == nil then EnKaiInv = {} end
	
	Command.Event.Attach(Event.Item.Slot, EnKai.internal.SlotUpdate, "EnKai.inventory.Item.Slot")
	Command.Event.Attach(Event.Item.Update, EnKai.internal.ItemUpdate, "EnKai.inventory.Item.Update")
	--Command.Event.Attach(Event.Interaction	, EnKai.internal.InterAction, "EnKai.inventory.Ineraction")	
	
	EnKai.eventHandlers["EnKai.InventoryManager"]["Update"], EnKai.events["EnKai.InventoryManager"]["Update"] = Utility.Event.Create(addonInfo.identifier, "EnKai.InventoryManagerUpdate")
	
	privateVars.invManager = true

end

function EnKai.internal.getInventory ()
	
	--if EnKaiInv[EnKai.internal.playerDetails.name] == nil then
		EnKaiInv[EnKai.internal.playerDetails.name] = { invDB = {}, invSlots = {} }
	--end

	local slots = { Inspect.Item.List(Utility.Item.Slot.Inventory()), Inspect.Item.List(Utility.Item.Slot.Bank()), Inspect.Item.List(Utility.Item.Slot.Vault()), Inspect.Item.List(Utility.Item.Slot.Quest()) }
	
	local lu = {}
	
	for idx = 1, #slots, 1 do
		for slot, key in pairs(slots[idx]) do
			if key ~= nil then
				lu[slot] = true
			end
		end
	end
			
	local itemList = Inspect.Item.Detail(lu)

	for key, v in pairs(itemList) do
		if v.stack == nil then v.stack = 1 end
	
		EnKaiInv[EnKai.internal.playerDetails.name].invSlots[key] = { id = v.type, stack = v.stack } 		
	
		if EnKaiInv[EnKai.internal.playerDetails.name].invDB[v.type] == nil then
			EnKaiInv[EnKai.internal.playerDetails.name].invDB[v.type] = v.stack
		else
			EnKaiInv[EnKai.internal.playerDetails.name].invDB[v.type] = EnKaiInv[EnKai.internal.playerDetails.name].invDB[v.type] + v.stack
		end
	end		
	
end

function EnKai.internal.processUpdate (updates)

	if EnKaiInv[EnKai.internal.playerDetails.name] == nil then EnKai.internal.getInventory () end
	
	local invSlots = EnKaiInv[EnKai.internal.playerDetails.name].invSlots
	local invDB = EnKaiInv[EnKai.internal.playerDetails.name].invDB
	
	local updatedKeys = {}
	
	for slot, key in pairs(updates) do
		if EnKai.strings.startsWith(slot, 'sg') == false then
	
			if key == "nil" then key = false end
		
			if invSlots[slot] ~= nil then
				updatedKeys[invSlots[slot].id] = -invSlots[slot].stack
				invDB[invSlots[slot].id] = invDB[invSlots[slot].id] - invSlots[slot].stack
			end
			
			if key == false then
				invSlots[slot] = nil
			else
				local details = Inspect.Item.Detail(slot)
				if details ~= nil then					
					
					if details.stack == nil then details.stack = 1 end
					invSlots[slot] = { id = details.type, stack = details.stack}
					
					updatedKeys[details.type] = details.stack
					
					if invDB[details.type] == nil then
						invDB[details.type] = details.stack
					else
						invDB[details.type] = invDB[details.type] + details.stack
					end
				end
			end
		end
	end
	
	EnKai.eventHandlers["EnKai.InventoryManager"]["Update"](updatedKeys)

end

function EnKai.internal.SlotUpdate (_, updates)
--	print ('--- slot update ----')
--	dump (updates)
	EnKai.internal.processUpdate (updates)

end

function EnKai.internal.ItemUpdate (_, updates)
--	print ('--- slot update ----')
--	dump (updates)
	EnKai.internal.processUpdate (updates)
	
end

function EnKai.internal.InterAction(_, interaction, state)

	if interaction == 'bank' and state == true then
		EnKai.internal.getInventory ()
	end

end

function EnKai.inventory.queryByKey (key)

	if privateVars.invManager == false then 
		EnKai.tools.error.display ("EnKai", "Inventory manager not initialzed", 1)
		return
	end

	if EnKaiInv[EnKai.internal.playerDetails.name] == nil then EnKai.internal.getInventory () end

	if EnKaiInv[EnKai.internal.playerDetails.name].invDB[key] ~= nil then return EnKaiInv[EnKai.internal.playerDetails.name].invDB[key] end
	
	if string.find(key, ',') ~= nil then	
		local details = Inspect.Item.Detail(key)
		if EnKaiInv[EnKai.internal.playerDetails.name].invDB[details.type] ~= nil then return EnKaiInv[EnKai.internal.playerDetails.name].invDB[details.type] end
	end
	
	return 0

end

function EnKai.inventory.getAvailableSlots()

	local availSlots = {}
	local allSlots = nil
	
	local slots = Inspect.Item.List(Utility.Item.Slot.Inventory())
	local initOk = false
	
	for slot, details in pairs (slots) do
		if EnKai.strings.startsWith(slot, "sibg.") == false then initOk = true end 		
		if details == false then table.insert (availSlots, slot) end
	end
	
	if initOk == false then return false end
	
	return availSlots

end

function EnKai.inventory.getQuestItems ()

	local slots = Inspect.Item.List(Utility.Item.Slot.Quest())
	local lu = {}
	
	for slot, key in pairs(slots) do
		if key ~= nil then lu[slot] = true end
	end
	
	return Inspect.Item.Detail(lu)		

end

 function EnKai.inventory.getQuestItemSlot (type)
 
 	local info = EnKai.inventory.getQuestItems ()
 	
 	for slot, details in pairs(info) do
 		if details.type == type then
 			return slot
 		end
 	end
 
 	return nil
 
 end 