local addonInfo, privateVars = ...

function EnKai.ui.nkGridCell(name, parent) 

	if EnKai.internal.checkEvents (name, true) == false then return nil end

	-- pre-define ui elements --
	
	local cell = UI.CreateFrame ('Canvas', name, parent)
	cell:SetWidth(0)
	cell:SetHeight(0)
	
	local label = UI.CreateFrame ('Text', name .. 'label', cell)	
	local texture = nil
	
	local properties = {}

	function cell:SetValue(property, value)
		properties[property] = value
	end
	
	function cell:GetValue(property)
		return properties[property]
	end
	
	-- default values --
	
	local borderColor = { 1, 1, 1, 1 }
	local bodyColor = { 0, 0, 0, 1 }
	local labelColor = { 1, 1, 1, 1 }
	local fontSize = 13
	local fontSizeMod = 6
	local height = fontSize + fontSizeMod
	local width = 50 
	local isTexture = false
	local orientation = "CENTERLEFT"
	local align = 'left'
	local textureHeight, textureWidth = nil, nil
	local stroke = {r = 1, g = 1, b = 1, a = 1, thickness = 1 }
	local path = {	{xProportional = 0, yProportional = 0}, 
						{xProportional = 0, yProportional = 1},
						{xProportional = 1, yProportional = 1},
						{xProportional = 1, yProportional = 0},
						{xProportional = 0, yProportional = 0} }
	local fill = {type = 'solid', r = 0, g = 0, b = 0, a = 1}
	
	-- fill ui elements with live --
	
	cell:SetWidth(width)
	cell:SetHeight(height)
	--cell:SetBackgroundColor(borderColor[1], borderColor[2], borderColor[3], borderColor[4])
	
	label:SetPoint ("CENTER", cell, "CENTER")
	label:SetFontColor (labelColor[1], labelColor[2], labelColor[3], labelColor[4])
	label:SetFontSize (fontSize)
	label:SetHeight (height - 2)
	if cell:GetWidth()-2 < label:GetWidth() then label:SetWidth(cell:GetWidth()-2) end
	
	function cell:SetText(text)		
		label:SetText(text)
		cell:SetAlign(align)
	end
	
	function cell:SetLabelColor (r, g, b, a)
		labelColor = {r, g, b, a}
		label:SetFontColor (r, g, b, a)
	end
	
	function cell:SetBorderColor(r, g, b, a)
		borderColor = {r, g, b, a}
		stroke = {r = r, g = g, b = b, a = a, thickness = 1 } 
		cell:SetShape(path, fill, stroke)
	end
	
	function cell:SetBodyColor(r, g, b, a)
		bodyColor = {r, g, b, a}
		fill = {type = 'solid', r = r, g = g, b = b, a = a} 
		cell:SetShape(path, fill, stroke)
	end
	
	-- texture handling
	
	function cell:IsTexture()
		return isTexture
	end
	
	function cell:SetTexture(textureType, texturePath)
		if texture == nil then
			label:SetVisible(false)
			texture = UI.CreateFrame("Texture", name .. 'texture', cell)		
			texture:SetPoint ("CENTER", cell, "CENTER")
			
			if textureWidth ~= nil or textureHeight ~= nil then
				local width = textureWidth or textureHeight
				local height = textureHeight or textureWidth
				
				if width > cell:GetWidth()-2 then 
					height = height * (width / (cell:GetWidth()-2))
					width = cell:GetWidth()-2
				end
				
				if height > cell:GetHeight()-2 then 
					width = width * (height / (cell:GetHeight()-2))
					height = cell:GetHeight()-2 
				end
			else
				if cell:GetWidth()-2 > cell:GetHeight()-2 then
					texture:SetHeight(cell:GetHeight()-2)
					texture:SetWidth(cell:GetHeight()-2)
				else
					texture:SetHeight(cell:GetWidth()-2)
					texture:SetWidth(cell:GetWidth()-2)
				end
			end
			
		end
		
		if texturePath == nil then textureType, texturePath = 'EnKai', 'gfx/empty.png' end
		texture:SetTextureAsync(textureType, texturePath)
		isTexture = true
			
	end
	
	function cell:SetTextureWidth(newWidth)
		textureWidth = newWidth
		texture:SetWidth(textureWidth)
	end
	
	function cell:SetTextureHeight(newHeight)
		textureHeight = newHeight
		texture:SetHeight(textureHeight)
	end
	
	-- layout options
	
	function cell:GetOrientation()
		return orientation
	end
	
	function cell:SetAlign(newAlign)
		align = newAlign
		if align == 'center' then
			orientation = "CENTER"
		else
			orientation = "CENTERLEFT"
		end
		
		if cell:IsTexture() then
			texture:ClearAll()
			texture:SetPoint (orientation, cell, orientation)
			texture:SetHeight(cell:GetHeight()-2)
			texture:SetWidth(cell:GetWidth()-2)
		else
			label:ClearAll()
			label:SetPoint (orientation, cell, orientation)
			if cell:GetWidth()-2 < label:GetWidth() then label:SetWidth(cell:GetWidth()-2) end
		end
	end
	
	function cell:SetFontSize(newFontSize)
		fontSize = newFontSize
		label:SetFontSize(fontSize)
		cell:SetHeight( fontSize + fontSizeMod)
	end
	
	-- replacing original functions
	
	local oSetWidth, oSetHeight = cell.SetWidth, cell.SetHeight
	
	function cell:SetWidth(newWidth)
		width = newWidth
		oSetWidth(self, width)
		if cell:GetWidth()-2 < label:GetWidth() then label:SetWidth(cell:GetWidth()-2) end
		cell:SetShape(path, fill, stroke)
	end
	
	function cell:SetHeight(newHeight)
		height = newHeight
		oSetHeight(self, height)
		label:SetHeight(height-2)
		cell:SetShape(path, fill, stroke)
	end
	
	local oSetVisible = cell.SetVisible
	
	function cell:SetVisible(flag)
		oSetVisible(self, flag)
		label:SetVisible(flag)
		if cell:IsTexture() then texture:SetVisible(flag) end
	end
	
	return cell
	
end
